% author: Matthias Humet
% version from 15-12-2010

function [ Ghat,Q ] = christoffel( G, alpha )
%CHRISTOFFEL Performs a QR step on a unitary Hessenberg matrix.
%   Let H    = G{1}*...*G{n},              n = length(G)
%   and Hhat = Ghat{1}*...*Ghat{n}
%
%  This function computes Ghat from G, such that
%       H    - alpha*I = Q*R
%       Hhat - alpha*I = R*Q
%  performing a bulge chasing algorithm using "shift-through".
%  I.e.
%   If [Q,R] = qr(H-alpha*eye(n+1))
%   Then
%       Hhat - alpha*eye(n+1) == R*Q.

x = G{1}(:,1) - [alpha; 0];

Q{1} = givens_alg(x(1),x(2));
M = Q{1}*G{1};

Q{1} = Q{1}';

n = length(G);

for k = 1:n-1 
    [Q{k+1},Ghat{k},M] = ST(M,G{k+1},Q{k});
end

Ghat{n} = M*Q{n}; 
% --> Ghat{1}*...Ghat{n} = Q{n}'*...*Q{1}'*(G{1}*...*G{n})*Q{1}*...*Q{n}

% calculate R_{n+1,n+1} with R the matrix 
%       R = Q{n}'* ... *Q{1}'*( G{1}*...*G{n} - alpha*eye(n+1) ) 
%    -> R = Q'*(H-alpha*I)
%
% R is de upper triangular reduction of (H-alpha*I) but with 
% R(end,end) =: Rnn complex.
%
% It follows that the element Hhat(n+1,n) = Ghat{n}(2,1) is complex. We
% need one more transformation.

if isa(alpha,'sym')
    if Ghat{n}(2,1) ~= 0
        Q{n+1} = diag([1, Ghat{n}(2,1)/abs(Ghat{n}(2,1))]);
    else
        Q{n+1} = diag([1, 0]);
    end
else
    Q{n+1} = diag([1, sign(double(Ghat{n}(2,1)))]);
end
% 
% % Apply Q{n+1} to make Rnn real (implicitly) and Ghat{n}(2,1) real.
Ghat{n} = Q{n+1}'*Ghat{n}*Q{n+1};

% ====================
% This works if the last G is also a reflection with det = -1.
%
% Ghat{n}(2,1) = abs(Ghat{n}(2,1));
% Ghat{n}(1,2) = abs(Ghat{n}(1,2));


end

