% author: Matthias Humet
% 
%
function [ G, beta ] = givensfactors( input )
%GIVENSFACTORS Compute Givens factors G from reflection coefficients or
%from unitary Hessenberg matrix.
%
%   G = givensfactors(r) 
%       where r is a vector of reclection coefficients (|r(k)|<1) will
%       compute the Givensfactors 
%           G{k} = [-r(k) rho; rho r(k)'], with rho=sqrt(1-|r(k)|^2)
%
%   [G,beta] = givensfactors(H) 
%       where H is a unitary Hessenberg matrix will compute the Givens
%       factors that constitute the unitary Hessenberg matrix H. Beta is a
%       scaling factor for the last column. 
%       I.e., 
%           H == unithess(G) * diag([ones(n-1,1) beta])


% input == H
if size(input,1) == size(input,2) && size(input,1) > 1
    H = input;
    [G,Glast] = fold_unithess(H);
    beta = Glast(2,2);
% input == r
elseif size(input,1) == 1 || size(input,2) == 1
    r = input;
    n = length(r);    
    for k = 1:n
        % check if r(k) is indeed a reflection coefficient
        rabs = abs(r(k));     
        rho = sqrt(1-rabs^2);
        if double(rabs) > 1
            warning(['*** givens_factors.m: |r(',int2str(k),')| > 1. Rho will be imaginary!'])
        end
        G{k} = [ -r(k) rho; rho r(k)' ];
    end   
else
    disp('** error givensfactors.m: Wrong input.')    
end

end

