% author: Matthias Humet
% 
%
function [ m, count] = valid_m_range( c, alpha, K, acc )
%VALID_M_RANGE
%   Given a sequence of moments c, and a complex number alpha outside the
%   unit circle, the maximum modulus of m for which uvarov(c,m,alpha) is
%   positive definite, and this for K different angles of m. The maximum
%   holds upto an accuracy of acc.
%
%   The K angles are taken equidistant over the unit circle.
%
%   INPUTS
%       c:          sequence of moments [c0,...,cn]
%       alpha:      complex number (outside unit circle, but probably works
%                       for all complex numbers) 
%       K:          [optional] number of angles, default is 100.
%       acc:        [optional] accuracy of maximum, default is 1e2.
%       

% first, check if toeplitz(c') > 0, else abort
if ~moments_posdef(c)
    warning(' ** valid_m_range: moment sequence c must be positive definite. [Aborted function]')
    m = 0;
    count = 0;
    return
end

if nargin > 2 && length(K) == 1
    L = K;
else
    L = 100;
end

if nargin > 3 && length(acc) == 1
    accuracy = acc;
else 
    accuracy = 1e-2;
end

maxit = 30;

% output
m = zeros(L,1);
count = zeros(L,1);

n = length(c);
theta = linspace(0,2*pi*(L-1)/L,L); %L = 1; theta = theta(55);
arg = exp(i*theta);

% startwaarde
emin = min(eig(toeplitz(c')));
% hack: emin should be > 0 because of first check above, but can be very
% small negative. In this case we take its absolute value.
if emin < 0
    fprintf('\n** emin was negative (emin = %0.3e), abs value is taken.\n',emin)
    emin = abs(emin);
end
    
mstart = emin/abs(alpha)^n;
assert(imag(mstart) < 1e-14);

fprintf( 'l: ')
for l = 1:L
    if mod(l,20) == 0
        fprintf('%3.2f,',l/L)
    end
    count(l) = 0;
    % TO DEBUG
%     if l == 27
%         n = n + 0;
%     end
    
    % First, determine interval
    %%fprintf('== \nInterval detection\n==\n')
    posdef = moments_posdef(uvarov_moments(c,mstart*arg(l),alpha));
    count(l) = count(l) + 1;
    if posdef
        m1 = mstart;
        m2 = m1;
        delta = mstart/10;
        %%fprintf('m2: ')
        while posdef
            %%fprintf('%4.3f, ',m2)
            m1 = m2;
            m2 = m2 + delta;
            posdef = moments_posdef(uvarov_moments(c,m2*arg(l),alpha));
            count(l) = count(l) + 1;
            delta = delta*2;
        end
        %%fprintf('\n')
    else
        m2 = mstart;
        m1 = m2;
        delta = mstart/8;
        %fprintf('m1: ')
        while ~posdef
            %fprintf('%4.3f, ',m1)
            m2 = m1;
            m1 = m1 - delta;
            posdef = moments_posdef(uvarov_moments(c,m1*arg(l),alpha));
            count(l) = count(l) + 1;
            delta = delta*2;
            if m1 < 0
                posdef = 1;
                m1 = 0;
            end
        end
        %fprintf('\n')
    end
    
    % Then, narrow down interval:
    %fprintf('\n==\n Nawrrowing down interval\n==\n')
    %fprintf('mnew: ')
    while (m2 - m1)/m1 > accuracy
        mnew = (m1+m2)/2;
        %fprintf('%4.3f, ',mnew)
        posdef = moments_posdef(uvarov_moments(c,mnew*arg(l),alpha));
        count(l) = count(l) + 1;
        if count(l) > maxit
            break
        end
        if posdef
            m1 = mnew;
        else
            m2 = mnew;
        end
    end
    %fprintf('\n')
    m(l) = m1 * arg(l);
    mstart = m1;
    if mstart < 0
        fprintf('l = %i: mstart = %4.4f\n',l,mstart)
    end
end
fprintf( '\n')
  
end